<?php

namespace Controller;

use Exception\CustomException;
use Util\Utilities;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\HTTP\Kernel;

class Server extends Simple {
    /** @var \Service\ProductDeploy $productDeployService */
    protected $serverService;

    public function __construct(\Service\Server $serverService) {
        $this->serverService = $serverService;
    }

    public function findAvailableCustomerServers(Request\Filter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $customerServer = $this->serverService->findAvailableCustomerServers($conditions);
            $response->addDataSet(new DataSet('findAvailableCustomerServers', $customerServer));
        } catch (\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }    

    public function findAllowedProfiles(Request\Filter $request, Response $response) {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $id  = Utilities::getValueInFilterDataSourceByName($conditions, 'SERVER_ID');
            $profiles = $this->serverService->findAllowedProfiles($id);
            $response->addDataSet(new DataSet('findAllowedProfiles', $profiles));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function saveAvailableCustomerServer(Request\DataSet $request, Response $response) {
        try {
            $customerServer = $request->getDataSet()->getRows()[0];
            $id = empty($customerServer['ID']) ? null : $customerServer['ID'];
            $name = empty($customerServer['NAME']) ? null : $customerServer['NAME'];
            $wsEmail = empty($customerServer['WSEMAIL']) ? null : $customerServer['WSEMAIL'];
            $backupLimit = empty($customerServer['BACKUP_LIMIT']) ? null : $customerServer['BACKUP_LIMIT'];
            $description = empty($customerServer['DESCRIPTION']) ? null : $customerServer['DESCRIPTION'];
            $isActive = empty($customerServer['IS_ACTIVE']) ? null : $customerServer['IS_ACTIVE'];
            $isWindows = empty($customerServer['IS_WINDOWS']) ? null : $customerServer['IS_WINDOWS'];
            $isRequiredTicket = empty($customerServer['IS_REQUIRED_TICKET']) ? "N" : $customerServer['IS_REQUIRED_TICKET'];
            $allowedProfiles = empty($customerServer['NRPERFILACESSO']) ? [] : $customerServer['NRPERFILACESSO'];
            $serverId = $this->serverService->saveAvailableCustomerServer($id, $name, $wsEmail, $backupLimit,
                $description, $isActive, $isWindows, $isRequiredTicket, $allowedProfiles);
            $response->addDataSet(new DataSet('savedServer', array($serverId)));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function removeAvailableCustomerServer(Request\DataSet $request, Response $response) {
        try {
            $customerServer = $request->getDataSet()->getRows()[0];
            $id = empty($customerServer['ID']) ? null : $customerServer['ID'];
            $this->serverService->removeAvailableCustomerServer($id);
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function validateServerPermissions(Request\Filter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $operation  = Utilities::getValueInFilterDataSourceByName($conditions, 'OPERATION');
            $serverId   = Utilities::getValueInFilterDataSourceByName($conditions, 'SERVER_ID');
            $productId  = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID');
            $needsApproval = $this->serverService->validateServerPermissions($operation, $serverId, $productId);
            $response->addDataSet(new DataSet('serverPermissions', $needsApproval));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function validateServerMnemonic(Request\Filter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $operation  = Utilities::getValueInFilterDataSourceByName($conditions, 'OPERATION');
            $serverId   = Utilities::getValueInFilterDataSourceByName($conditions, 'SERVER_ID');
            $productIds = Utilities::getValueInFilterDataSourceByName($conditions, 'PRODUCT_ID');
            $serverMnemonic = array();
            $productsId = array();
            if(gettype($productIds) != "array"){
                $productIds = array($productIds);
            }
            foreach($productIds as $productId) {
                $this->serverService->validateServerMnemonic($operation, $serverId, $productId, $serverMnemonic, $productsId);
            }
            if($serverMnemonic){
                $this->serverService->validateServerProductRelationship($productsId, $serverMnemonic, $serverId);
            };
            $response->addDataSet(new DataSet('serverMnemonic', $serverMnemonic));
        } catch(\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
        
    }

    public function findMnemonicByServer(Request\Filter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $serverId   = Utilities::getValueInFilterDataSourceByName($conditions,'SERVER_ID', true);
            $productIds = Utilities::getValueInFilterDataSourceByName($conditions,'PRODUCTS_ID', true);
            $mnemonics  = $this->serverService->findMnemonicByServer($serverId, $productIds, $conditions);
            $response->addDataSet(new DataSet('findMnemonicByServer', $mnemonics));
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
    
    public function findMnemonicGrouping(Request\Filter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $page       = $request->getFilterCriteria()->getPage();
            $pageSize   = $request->getFilterCriteria()->getPageSize();
            $mnemonics  = $this->serverService->findMnemonicGrouping($conditions, $page, $pageSize);
            $response->addDataSet(new DataSet('findMnemonicGrouping', $mnemonics));
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findProductsByServer(Request\Filter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $serverId   = Utilities::getValueInFilterDataSourceByName($conditions,'SERVER_ID', true);
            $products   = $this->serverService->findProductsByServer($serverId, $conditions);
            $response->addDataSet(new DataSet('products', $products));
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
            $response->setError(new Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function saveMnemonicByServer(Request\DataSet $request, Response $response) {
        try {
            $mnemonic = $request->getDataSet()->getRows()[0];
            $mnemonicId     = $mnemonic['MNEMONIC_ID'];
            $mnemonicString = $mnemonic['MNEMONIC'];
            $serverId       = $mnemonic['SERVER_ID'];
            $value          = $mnemonic['VALUE'];
            $label          = $mnemonic['LABEL'];
            $isEncrypted    = $mnemonic['IS_ENCRYPTED'];
            $mnemonicGroup    = empty($mnemonic['MNEMONIC_GROUP_ID']) ? null : $mnemonic['MNEMONIC_GROUP_ID'];
            $savedMnemonic  = $this->serverService->saveMnemonicByServer($serverId, $mnemonicId, $mnemonicString,
                $value, $label, $isEncrypted, $mnemonicGroup);
            $response->addDataSet(new DataSet('savedMnemonic', array($savedMnemonic)));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }
} 
